@@ -186,10 +186,8 @@ done:
 }
 
 static gboolean
-gsm_new_auto_connection (NMDevice *device,
-                         gpointer dclass_data,
-                         AppletNewAutoConnectionCallback callback,
-                         gpointer callback_data)
+do_mobile_wizard (AppletNewAutoConnectionCallback callback,
+                  gpointer callback_data)
 {
 	MobileWizard *wizard;
 	AutoGsmWizardInfo *info;
@@ -216,6 +214,70 @@ gsm_new_auto_connection (NMDevice *device,
 	return TRUE;
 }
 
+static gboolean
+gsm_new_auto_connection (NMDevice *device,
+                         gpointer dclass_data,
+                         AppletNewAutoConnectionCallback callback,
+                         gpointer callback_data)
+{
+	return do_mobile_wizard (callback, callback_data);
+}
+
+static void
+dbus_3g_add_and_activate_cb (NMClient *client,
+                             NMActiveConnection *active,
+                             const char *connection_path,
+                             GError *error,
+                             gpointer user_data)
+{
+	if (error)
+		g_warning ("Failed to add/activate connection: (%d) %s", error->code, error->message);
+}
+
+typedef struct {
+	NMApplet *applet;
+	NMDevice *device;
+} Dbus3gInfo;
+
+static void
+dbus_connect_3g_cb (NMConnection *connection,
+                    gboolean auto_created,
+                    gboolean canceled,
+                    gpointer user_data)
+{
+	Dbus3gInfo *info = user_data;
+
+	if (canceled == FALSE) {
+		g_return_if_fail (connection != NULL);
+
+		/* Ask NM to add the new connection and activate it; NM will fill in the
+		 * missing details based on the specific object and the device.
+		 */
+		nm_client_add_and_activate_connection (info->applet->nm_client,
+		                                       connection,
+		                                       info->device,
+		                                       "/",
+		                                       dbus_3g_add_and_activate_cb,
+		                                       info->applet);
+	}
+
+	g_object_unref (info->device);
+	memset (info, 0, sizeof (*info));
+	g_free (info);
+}
+
+void
+applet_gsm_connect_network (NMApplet *applet, NMDevice *device)
+{
+	Dbus3gInfo *info;
+
+	info = g_malloc0 (sizeof (*info));
+	info->applet = applet;
+	info->device = g_object_ref (device);
+
+	do_mobile_wizard (dbus_connect_3g_cb, info);
+}
+
 static void
 gsm_menu_item_activate (GtkMenuItem *item, gpointer user_data)
 {
@@ -345,11 +407,12 @@ gsm_add_menu_item (NMDevice *device,
 		item = nm_mb_menu_item_new (nm_setting_connection_get_id (s_con),
 		                            info->quality_valid ? info->quality : 0,
 		                            info->op_name,
+		                            TRUE,
 		                            gsm_act_to_mb_act (info),
 		                            gsm_state_to_mb_state (info),
 		                            info->modem_enabled,
 		                            applet);
-
+		gtk_widget_set_sensitive (GTK_WIDGET (item), TRUE);
 		add_connection_item (device, active, item, menu, applet);
 	}
 
@@ -361,13 +424,16 @@ gsm_add_menu_item (NMDevice *device,
 			gtk_widget_show (item);
 		}
 	} else {
+		/* Otherwise show idle registration state or disabled */
 		item = nm_mb_menu_item_new (NULL,
 		                            info->quality_valid ? info->quality : 0,
 		                            info->op_name,
+		                            FALSE,
 		                            gsm_act_to_mb_act (info),
 		                            gsm_state_to_mb_state (info),
 		                            info->modem_enabled,
 		                            applet);
+		gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
 
@@ -782,7 +848,7 @@ unlock_dialog_response (GtkDialog *dialog,
 	if (unlock_code == UNLOCK_CODE_PIN) {
 		dbus_g_proxy_begin_call_with_timeout (info->card_proxy, "SendPin",
 		                                      unlock_pin_reply, info, NULL,
-		                                      12000,  /* 12 seconds */
+		                                      15000,  /* 15 seconds */
 		                                      G_TYPE_STRING, code1,
 		                                      G_TYPE_INVALID);
 	} else if (unlock_code == UNLOCK_CODE_PUK) {
@@ -795,7 +861,7 @@ unlock_dialog_response (GtkDialog *dialog,
 
 		dbus_g_proxy_begin_call_with_timeout (info->card_proxy, "SendPuk",
 		                                      unlock_puk_reply, info, NULL,
-		                                      12000,  /* 12 seconds */
+		                                      15000,  /* 15 seconds */
 		                                      G_TYPE_STRING, code1,
 		                                      G_TYPE_STRING, code2,
 		                                      G_TYPE_INVALID);
@@ -975,7 +1041,7 @@ find_provider_for_mcc_mnc (GHashTable *table, const char *mccmnc)
 
 	if (name3)
 		return g_strdup (name3);
-	return name2 ? g_strdup (name2) : NULL;
+	return g_strdup (name2);
 }
 
 static char *
