@@ -20,6 +20,8 @@
  * (C) Copyright 2008 Red Hat, Inc.
  */
 
+#include "config.h"
+
 #include <string.h>
 #include <glib.h>
 #include <gmodule.h>
@@ -212,13 +214,11 @@ import_vpn_from_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
 	g_hash_table_foreach (plugins, try_import, (gpointer) &import_info);
 
 	connection = import_info.connection;
-	if (connection) {
-		if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_UNKNOWN)
-			nm_connection_set_scope (connection, NM_CONNECTION_SCOPE_USER);
+	if (connection)
 		info->callback (connection, info->user_data);
-	} else {
+	else {
 		GtkWidget *err_dialog;
-		char *basename = g_path_get_basename (filename);
+		char *bname = g_path_get_basename (filename);
 
 		err_dialog = gtk_message_dialog_new (NULL,
 		                                     GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -227,8 +227,8 @@ import_vpn_from_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
 		                                     _("Cannot import VPN connection"));
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
 		                                 _("The file '%s' could not be read or does not contain recognized VPN connection information\n\nError: %s."),
-		                                 basename, import_info.error ? import_info.error->message : "unknown error");
-		g_free (basename);
+		                                 bname, import_info.error ? import_info.error->message : "unknown error");
+		g_free (bname);
 		g_signal_connect (err_dialog, "delete-event", G_CALLBACK (gtk_widget_destroy), NULL);
 		g_signal_connect (err_dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
 		gtk_widget_show_all (err_dialog);
@@ -257,7 +257,6 @@ vpn_import (VpnImportSuccessCallback callback, gpointer user_data)
 {
 	GtkWidget *dialog;
 	ActionInfo *info;
-	const char *home_folder;
 
 	dialog = gtk_file_chooser_dialog_new (_("Select file to import"),
 	                                      NULL,
@@ -265,9 +264,6 @@ vpn_import (VpnImportSuccessCallback callback, gpointer user_data)
 	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 	                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 	                                      NULL);
-        home_folder = g_get_home_dir ();
-        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), home_folder);
-
 	info = g_malloc0 (sizeof (ActionInfo));
 	info->callback = callback;
 	info->user_data = user_data;
@@ -303,19 +299,19 @@ export_vpn_to_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
 	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
 		int replace_response;
 		GtkWidget *replace_dialog;
-		char *basename;
+		char *bname;
 
-		basename = g_path_get_basename (filename);
+		bname = g_path_get_basename (filename);
 		replace_dialog = gtk_message_dialog_new (NULL,
 		                                         GTK_DIALOG_DESTROY_WITH_PARENT,
 		                                         GTK_MESSAGE_QUESTION,
 		                                         GTK_BUTTONS_CANCEL,
 		                                         _("A file named \"%s\" already exists."),
-		                                         basename);
+		                                         bname);
 		gtk_dialog_add_buttons (GTK_DIALOG (replace_dialog), _("_Replace"), GTK_RESPONSE_OK, NULL);
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (replace_dialog),
-							  _("Do you want to replace %s with the VPN connection you are saving?"), basename);
-		g_free (basename);
+							  _("Do you want to replace %s with the VPN connection you are saving?"), bname);
+		g_free (bname);
 		replace_response = gtk_dialog_run (GTK_DIALOG (replace_dialog));
 		gtk_widget_destroy (replace_dialog);
 		if (replace_response != GTK_RESPONSE_OK)
@@ -344,7 +340,7 @@ export_vpn_to_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
 done:
 	if (!success) {
 		GtkWidget *err_dialog;
-		char *basename = filename ? g_path_get_basename (filename) : g_strdup ("(none)");
+		char *bname = filename ? g_path_get_basename (filename) : g_strdup ("(none)");
 
 		err_dialog = gtk_message_dialog_new (NULL,
 		                                     GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -353,8 +349,8 @@ done:
 		                                     _("Cannot export VPN connection"));
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
 		                                 _("The VPN connection '%s' could not be exported to %s.\n\nError: %s."),
-		                                 id ? id : "(unknown)", basename, error ? error->message : "unknown error");
-		g_free (basename);
+		                                 id ? id : "(unknown)", bname, error ? error->message : "unknown error");
+		g_free (bname);
 		g_signal_connect (err_dialog, "delete-event", G_CALLBACK (gtk_widget_destroy), NULL);
 		g_signal_connect (err_dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
 		gtk_widget_show_all (err_dialog);
@@ -377,7 +373,6 @@ vpn_export (NMConnection *connection)
 	NMVpnPluginUiInterface *plugin;
 	NMSettingVPN *s_vpn = NULL;
 	const char *service_type;
-	const char *home_folder;
 
 	s_vpn = NM_SETTING_VPN (nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN));
 	service_type = s_vpn ? nm_setting_vpn_get_service_type (s_vpn) : NULL;
@@ -393,8 +388,6 @@ vpn_export (NMConnection *connection)
 	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 	                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 	                                      NULL);
-	home_folder = g_get_home_dir ();
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), home_folder);
 
 	plugin = vpn_get_plugin_by_service (service_type);
 	if (plugin) {
