@@ -1897,6 +1897,8 @@ nm_gconf_migrate_0_7_ca_cert_ignore (GConfClient *client)
 		                          &ignore_ca_cert);
 		if (ignore_ca_cert)
 			_set_ignore_ca_cert (client, uuid, FALSE);
+		g_free (uuid);
+
 		/* delete old key */
 		unset_one_setting_property (client, dir,
 		                            NM_SETTING_802_1X_SETTING_NAME,
@@ -2024,13 +2026,16 @@ nm_gconf_migrate_0_7_certs (GConfClient *client)
 #define NM_VPNC_PW_TYPE_UNUSED "unused"
 
 static NMSettingSecretFlags
-vpnc_type_to_flag (const char *vpnc_type)
+vpnc_type_to_flag (NMSettingVPN *s_vpn, const char *type_key)
 {
-	if (g_strcmp0 (vpnc_type, NM_VPNC_PW_TYPE_SAVE) == 0)
+	const char *tmp;
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, type_key);
+	if (g_strcmp0 (tmp, NM_VPNC_PW_TYPE_SAVE) == 0)
 		return NM_SETTING_SECRET_FLAG_NONE;
-	if (g_strcmp0 (vpnc_type, NM_VPNC_PW_TYPE_ASK) == 0)
+	if (g_strcmp0 (tmp, NM_VPNC_PW_TYPE_ASK) == 0)
 		return NM_SETTING_SECRET_FLAG_NOT_SAVED;
-	if (g_strcmp0 (vpnc_type, NM_VPNC_PW_TYPE_UNUSED) == 0)
+	if (g_strcmp0 (tmp, NM_VPNC_PW_TYPE_UNUSED) == 0)
 		return NM_SETTING_SECRET_FLAG_NOT_REQUIRED;
 	return NM_SETTING_SECRET_FLAG_NONE;
 }
@@ -2041,6 +2046,138 @@ vpnc_type_to_flag (const char *vpnc_type)
 #define NM_VPNC_KEY_XAUTH_PASSWORD "Xauth password"
 #define NM_VPNC_KEY_XAUTH_PASSWORD_TYPE "xauth-password-type"
 
+static void
+migrate_vpnc (NMConnection *connection, NMSettingVPN *s_vpn)
+{
+	NMSettingSecretFlags flags;
+
+	flags = NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+	flags |= vpnc_type_to_flag (s_vpn, NM_VPNC_KEY_SECRET_TYPE);
+	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_SECRET, flags, NULL);
+
+	flags = NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+	flags |= vpnc_type_to_flag (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
+	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_XAUTH_PASSWORD, flags, NULL);
+}
+
+#define NM_DBUS_SERVICE_OPENVPN "org.freedesktop.NetworkManager.openvpn"
+#define NM_OPENVPN_KEY_PASSWORD "password"
+#define NM_OPENVPN_KEY_CERTPASS "cert-pass"
+#define NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD "http-proxy-password"
+#define NM_OPENVPN_KEY_CONNECTION_TYPE "connection-type"
+#define NM_OPENVPN_CONTYPE_TLS          "tls"
+#define NM_OPENVPN_CONTYPE_PASSWORD     "password"
+#define NM_OPENVPN_CONTYPE_PASSWORD_TLS "password-tls"
+#define NM_OPENVPN_KEY_PROXY_TYPE "proxy-type"
+
+static NMSettingSecretFlags
+openvpn_get_secret_flags (const char *uuid, const char *secret_name)
+{
+	GList *found = NULL;
+	GnomeKeyringResult ret;
+	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+
+	ret = gnome_keyring_find_itemsv_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
+		                                  &found,
+		                                  KEYRING_UUID_TAG,
+		                                  GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+										  uuid,
+		                                  KEYRING_SN_TAG,
+		                                  GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+		                                  NM_SETTING_VPN_SETTING_NAME,
+		                                  KEYRING_SK_TAG,
+		                                  GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+		                                  secret_name,
+		                                  NULL);
+	if (ret != GNOME_KEYRING_RESULT_OK || found == NULL)
+		flags |= NM_SETTING_SECRET_FLAG_NOT_SAVED;
+	gnome_keyring_found_list_free (found);
+
+	return flags;
+}
+
+static void
+migrate_openvpn (NMConnection *connection, NMSettingVPN *s_vpn)
+{
+	NMSettingSecretFlags flags;
+	const char *tmp;
+	gboolean check_pw = FALSE, check_cp = FALSE;
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
+	if (!tmp)
+		return;
+
+	if (!strcmp (tmp, NM_OPENVPN_CONTYPE_TLS))
+		check_cp = TRUE;
+	else if (!strcmp (tmp, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
+		check_pw = TRUE;
+		check_cp = TRUE;
+	} else if (!strcmp (tmp, NM_OPENVPN_CONTYPE_PASSWORD))
+		check_pw = TRUE;
+
+	/* For each secret, we need to check the keyring to see whether the secret
+	 * is present or not, and if it's *not*, then we mark the secret as both
+	 * not-saved and agent-owned.  If it is present, the secret is just marked
+	 * agent-owned.
+	 */
+
+	if (check_pw) {
+		flags = openvpn_get_secret_flags (nm_connection_get_uuid (connection), NM_OPENVPN_KEY_PASSWORD);
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_PASSWORD, flags, NULL);
+	}
+
+	if (check_cp) {
+		flags = openvpn_get_secret_flags (nm_connection_get_uuid (connection), NM_OPENVPN_KEY_CERTPASS);
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_CERTPASS, flags, NULL);
+	}
+
+	/* HTTP proxy password */
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PROXY_TYPE);
+	if (g_strcmp0 (tmp, "http") == 0 || g_strcmp0 (tmp, "socks") == 0) {
+		flags = openvpn_get_secret_flags (nm_connection_get_uuid (connection), NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD);
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD, flags, NULL);
+	}
+}
+
+#define NM_OPENSWAN_PW_TYPE_SAVE   "save"
+#define NM_OPENSWAN_PW_TYPE_ASK    "ask"
+#define NM_OPENSWAN_PW_TYPE_UNUSED "unused"
+
+static NMSettingSecretFlags
+openswan_type_to_flag (NMSettingVPN *s_vpn, const char *flags_key)
+{
+	const char *tmp;
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, flags_key);
+	if (g_strcmp0 (tmp, NM_OPENSWAN_PW_TYPE_SAVE) == 0)
+		return NM_SETTING_SECRET_FLAG_NONE;
+	if (g_strcmp0 (tmp, NM_OPENSWAN_PW_TYPE_ASK) == 0)
+		return NM_SETTING_SECRET_FLAG_NOT_SAVED;
+	if (g_strcmp0 (tmp, NM_OPENSWAN_PW_TYPE_UNUSED) == 0)
+		return NM_SETTING_SECRET_FLAG_NOT_REQUIRED;
+	return NM_SETTING_SECRET_FLAG_NONE;
+}
+
+#define NM_DBUS_SERVICE_OPENSWAN "org.freedesktop.NetworkManager.openswan"
+#define NM_OPENSWAN_PSK_VALUE "pskvalue"
+#define NM_OPENSWAN_PSK_INPUT_MODES "pskinputmodes"
+#define NM_OPENSWAN_XAUTH_PASSWORD "xauthpassword"
+#define NM_OPENSWAN_XAUTH_PASSWORD_INPUT_MODES "xauthpasswordinputmodes"
+
+static void
+migrate_openswan (NMConnection *connection, NMSettingVPN *s_vpn)
+{
+	NMSettingSecretFlags flags;
+
+	flags = NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+	flags |= openswan_type_to_flag (s_vpn, NM_OPENSWAN_PSK_INPUT_MODES);
+	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENSWAN_PSK_VALUE, flags, NULL);
+
+	flags = NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+	flags |= openswan_type_to_flag (s_vpn, NM_OPENSWAN_XAUTH_PASSWORD_INPUT_MODES);
+	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENSWAN_XAUTH_PASSWORD, flags, NULL);
+}
+
 #define NM_DBUS_SERVICE_PPTP "org.freedesktop.NetworkManager.pptp"
 #define NM_PPTP_KEY_PASSWORD "password"
 
@@ -2063,29 +2200,23 @@ nm_gconf_migrate_09_secret_flags (GConfClient *client,
 
 	/* Migrate various VPN secret flags */
 	if (NM_IS_SETTING_VPN (setting)) {
-		NMSettingSecretFlags flags;
 		NMSettingVPN *s_vpn = NM_SETTING_VPN (setting);
-		const char *tmp, *service;
+		const char *service;
 
 		service = nm_setting_vpn_get_service_type (s_vpn);
 		if (g_strcmp0 (service, NM_DBUS_SERVICE_VPNC) == 0) {
-			/* vpnc stuff */
-			tmp = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_SECRET_TYPE);
-			if (tmp) {
-				flags = vpnc_type_to_flag (tmp) | NM_SETTING_SECRET_FLAG_AGENT_OWNED;
-				nm_setting_set_secret_flags (setting, NM_VPNC_KEY_SECRET, flags, NULL);
-			}
-
-			tmp = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
-			if (tmp) {
-				flags = vpnc_type_to_flag (tmp) | NM_SETTING_SECRET_FLAG_AGENT_OWNED;
-				nm_setting_set_secret_flags (setting, NM_VPNC_KEY_XAUTH_PASSWORD, flags, NULL);
-			}
+			migrate_vpnc (connection, s_vpn);
 			return;
 		} else if (g_strcmp0 (service, NM_DBUS_SERVICE_PPTP) == 0) {
 			/* Mark the password as agent-owned */
 			nm_setting_set_secret_flags (setting, NM_PPTP_KEY_PASSWORD, NM_SETTING_SECRET_FLAG_AGENT_OWNED, NULL);
 			return;
+		} else if (g_strcmp0 (service, NM_DBUS_SERVICE_OPENVPN) == 0) {
+			migrate_openvpn (connection, s_vpn);
+			return;
+		} else if (g_strcmp0 (service, NM_DBUS_SERVICE_OPENSWAN) == 0) {
+			migrate_openswan (connection, s_vpn);
+			return;
 		}
 
 		/* Other VPNs not handled specially here just go through the
