@@ -24,8 +24,6 @@
 #include <nm-setting-wireless.h>
 
 #include "wireless-security.h"
-#include "utils.h"
-#include "gconf-helpers.h"
 #include "helpers.h"
 
 struct _WirelessSecurityLEAP {
@@ -120,7 +118,7 @@ update_secrets (WirelessSecurity *parent, NMConnection *connection)
 }
 
 WirelessSecurityLEAP *
-ws_leap_new (NMConnection *connection, gboolean secrets_only)
+ws_leap_new (NMConnection *connection)
 {
 	WirelessSecurity *parent;
 	WirelessSecurityLEAP *sec;
@@ -169,9 +167,6 @@ ws_leap_new (NMConnection *connection, gboolean secrets_only)
 	if (wsec)
 		gtk_entry_set_text (GTK_ENTRY (widget), nm_setting_wireless_security_get_leap_username (wsec));
 
-	if (secrets_only)
-		gtk_widget_hide (widget);
-
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "show_checkbutton_leap"));
 	g_assert (widget);
 	g_signal_connect (G_OBJECT (widget), "toggled",
